@@ -53,6 +53,8 @@ typedef struct
 
 struct _GfMonitorMode
 {
+  GfMonitor         *monitor;
+
   gchar             *id;
   GfMonitorModeSpec  spec;
   GfMonitorCrtcMode *crtc_modes;
@@ -229,6 +231,8 @@ gboolean            gf_monitor_mode_foreach_output        (GfMonitor
                                                            gpointer                    user_data,
                                                            GError                    **error);
 
+gboolean            gf_monitor_mode_should_be_advertised  (GfMonitorMode              *monitor_mode);
+
 gboolean            gf_verify_monitor_mode_spec           (GfMonitorModeSpec          *mode_spec,
                                                            GError                    **error);
 
