@@ -33,44 +33,36 @@
 #include "gvc-mixer-control.h"
 
 #define SCALE_SIZE 128
-#define ADJUSTMENT_MAX_NORMAL gvc_mixer_control_get_vol_max_norm(NULL)
-#define ADJUSTMENT_MAX_AMPLIFIED gvc_mixer_control_get_vol_max_amplified(NULL)
-#define ADJUSTMENT_MAX (bar->priv->is_amplified ? ADJUSTMENT_MAX_AMPLIFIED : ADJUSTMENT_MAX_NORMAL)
-#define SCROLLSTEP (ADJUSTMENT_MAX / 100.0 * 5.0)
 
 struct GvcChannelBarPrivate
 {
-        GtkOrientation orientation;
-        GtkWidget     *scale_box;
-        GtkWidget     *start_box;
-        GtkWidget     *end_box;
-        GtkWidget     *image;
-        GtkWidget     *low_image;
-        GtkWidget     *scale;
-        GtkWidget     *high_image;
-        GtkAdjustment *adjustment;
-        GtkAdjustment *zero_adjustment;
-        gboolean       is_muted;
-        char          *icon_name;
-        char          *low_icon_name;
-        char          *high_icon_name;
-        GtkSizeGroup  *size_group;
-        gboolean       symmetric;
-        gboolean       click_lock;
-        gboolean       is_amplified;
-        guint32        base_volume;
+        GvcMixerControl *mixer_control;
+
+        GtkOrientation   orientation;
+        GtkWidget       *scale_box;
+        GtkWidget       *start_box;
+        GtkWidget       *end_box;
+        GtkWidget       *image;
+        GtkWidget       *low_image;
+        GtkWidget       *scale;
+        GtkWidget       *high_image;
+        GtkAdjustment   *adjustment;
+        GtkAdjustment   *zero_adjustment;
+        gboolean         is_muted;
+        char            *icon_name;
+        char            *low_icon_name;
+        char            *high_icon_name;
+        gboolean         click_lock;
+        gboolean         is_amplified;
+        guint32          base_volume;
 };
 
 enum
 {
         PROP_0,
+        PROP_MIXER_CONTROL,
         PROP_ORIENTATION,
         PROP_IS_MUTED,
-        PROP_ADJUSTMENT,
-        PROP_ICON_NAME,
-        PROP_LOW_ICON_NAME,
-        PROP_HIGH_ICON_NAME,
-        PROP_IS_AMPLIFIED
 };
 
 static void     gvc_channel_bar_finalize      (GObject            *object);
@@ -152,14 +144,6 @@ _scale_box_new (GvcChannelBar *bar)
         g_signal_connect (G_OBJECT (bar->priv->scale), "scroll-event",
                           G_CALLBACK (on_scale_scroll_event), bar);
 
-        if (bar->priv->size_group != NULL) {
-                gtk_size_group_add_widget (bar->priv->size_group, sbox);
-
-                if (bar->priv->symmetric) {
-                        gtk_size_group_add_widget (bar->priv->size_group, ebox);
-                }
-        }
-
         gtk_scale_set_draw_value (GTK_SCALE (priv->scale), FALSE);
 
         return box;
@@ -221,28 +205,6 @@ update_layout (GvcChannelBar *bar)
         gtk_widget_show_all (frame);
 }
 
-void
-gvc_channel_bar_set_size_group (GvcChannelBar *bar,
-                                GtkSizeGroup  *group,
-                                gboolean       symmetric)
-{
-        g_return_if_fail (GVC_IS_CHANNEL_BAR (bar));
-
-        bar->priv->size_group = group;
-        bar->priv->symmetric = symmetric;
-
-        if (bar->priv->size_group != NULL) {
-                gtk_size_group_add_widget (bar->priv->size_group,
-                                           bar->priv->start_box);
-
-                if (bar->priv->symmetric) {
-                        gtk_size_group_add_widget (bar->priv->size_group,
-                                                   bar->priv->end_box);
-                }
-        }
-        gtk_widget_queue_draw (GTK_WIDGET (bar));
-}
-
 void
 gvc_channel_bar_set_icon_name (GvcChannelBar  *bar,
                                const char     *name)
@@ -252,7 +214,6 @@ gvc_channel_bar_set_icon_name (GvcChannelBar  *bar,
         g_free (bar->priv->icon_name);
         bar->priv->icon_name = g_strdup (name);
         update_image (bar);
-        g_object_notify (G_OBJECT (bar), "icon-name");
 }
 
 void
@@ -267,7 +228,6 @@ gvc_channel_bar_set_low_icon_name   (GvcChannelBar *bar,
                 gtk_image_set_from_icon_name (GTK_IMAGE (bar->priv->low_image),
                                               bar->priv->low_icon_name,
                                               GTK_ICON_SIZE_MENU);
-                g_object_notify (G_OBJECT (bar), "low-icon-name");
         }
 }
 
@@ -283,7 +243,6 @@ gvc_channel_bar_set_high_icon_name  (GvcChannelBar *bar,
                 gtk_image_set_from_icon_name (GTK_IMAGE (bar->priv->high_image),
                                               bar->priv->high_icon_name,
                                               GTK_ICON_SIZE_MENU);
-                g_object_notify (G_OBJECT (bar), "high-icon-name");
         }
 }
 
@@ -300,25 +259,6 @@ gvc_channel_bar_set_orientation (GvcChannelBar  *bar,
         }
 }
 
-static void
-gvc_channel_bar_set_adjustment (GvcChannelBar *bar,
-                                GtkAdjustment *adjustment)
-{
-        g_return_if_fail (GVC_CHANNEL_BAR (bar));
-        g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));
-
-        if (bar->priv->adjustment != NULL) {
-                g_object_unref (bar->priv->adjustment);
-        }
-        bar->priv->adjustment = g_object_ref_sink (adjustment);
-
-        if (bar->priv->scale != NULL) {
-                gtk_range_set_adjustment (GTK_RANGE (bar->priv->scale), adjustment);
-        }
-
-        g_object_notify (G_OBJECT (bar), "adjustment");
-}
-
 GtkAdjustment *
 gvc_channel_bar_get_adjustment (GvcChannelBar *bar)
 {
@@ -372,6 +312,8 @@ gvc_channel_bar_scroll (GvcChannelBar *bar, GdkEventScroll *event)
         gdouble value;
         GdkScrollDirection direction;
         gdouble dx, dy;
+        gdouble adjustment_max;
+        gdouble scrollstep;
 
         g_return_val_if_fail (bar != NULL, FALSE);
         g_return_val_if_fail (GVC_IS_CHANNEL_BAR (bar), FALSE);
@@ -425,16 +367,23 @@ gvc_channel_bar_scroll (GvcChannelBar *bar, GdkEventScroll *event)
 
         value = gtk_adjustment_get_value (adj);
 
+        if (bar->priv->is_amplified)
+                adjustment_max = gvc_mixer_control_get_vol_max_amplified (bar->priv->mixer_control);
+        else
+                adjustment_max = gvc_mixer_control_get_vol_max_norm (bar->priv->mixer_control);
+
+        scrollstep = (adjustment_max / 100.0 * 5.0);
+
         if (dy > 0) {
-                if (value + dy * SCROLLSTEP > ADJUSTMENT_MAX)
-                        value = ADJUSTMENT_MAX;
+                if (value + dy * scrollstep > adjustment_max)
+                        value = adjustment_max;
                 else
-                        value = value + dy * SCROLLSTEP;
+                        value = value + dy * scrollstep;
         } else if (dy < 0) {
-                if (value + dy * SCROLLSTEP < 0)
+                if (value + dy * scrollstep < 0)
                         value = 0.0;
                 else
-                        value = value + dy * SCROLLSTEP;
+                        value = value + dy * scrollstep;
         }
 
         gvc_channel_bar_set_is_muted (bar, (value == 0.0));
@@ -521,28 +470,38 @@ gvc_channel_bar_get_is_muted  (GvcChannelBar *bar)
 void
 gvc_channel_bar_set_is_amplified (GvcChannelBar *bar, gboolean amplified)
 {
+        gdouble vol_max_normal;
+        gdouble adjustment_max;
+
         g_return_if_fail (GVC_IS_CHANNEL_BAR (bar));
 
+        vol_max_normal = gvc_mixer_control_get_vol_max_norm (bar->priv->mixer_control);
+
+        if (amplified)
+                adjustment_max = gvc_mixer_control_get_vol_max_amplified (bar->priv->mixer_control);
+        else
+                adjustment_max = gvc_mixer_control_get_vol_max_norm (bar->priv->mixer_control);
+
         bar->priv->is_amplified = amplified;
-        gtk_adjustment_set_upper (bar->priv->adjustment, ADJUSTMENT_MAX);
-        gtk_adjustment_set_upper (bar->priv->zero_adjustment, ADJUSTMENT_MAX);
+        gtk_adjustment_set_upper (bar->priv->adjustment, adjustment_max);
+        gtk_adjustment_set_upper (bar->priv->zero_adjustment, adjustment_max);
         gtk_scale_clear_marks (GTK_SCALE (bar->priv->scale));
 
         if (amplified) {
                 char *str;
 
-                if (bar->priv->base_volume == ADJUSTMENT_MAX_NORMAL) {
+                if (bar->priv->base_volume == vol_max_normal) {
                         str = g_strdup_printf ("<small>%s</small>", C_("volume", "100%"));
-                        gtk_scale_add_mark (GTK_SCALE (bar->priv->scale), ADJUSTMENT_MAX_NORMAL,
+                        gtk_scale_add_mark (GTK_SCALE (bar->priv->scale), vol_max_normal,
                                             GTK_POS_BOTTOM, str);
                 } else {
                         str = g_strdup_printf ("<small>%s</small>", C_("volume", "Unamplified"));
                         gtk_scale_add_mark (GTK_SCALE (bar->priv->scale), bar->priv->base_volume,
                                             GTK_POS_BOTTOM, str);
                         /* Only show 100% if it's higher than the base volume */
-                        if (bar->priv->base_volume < ADJUSTMENT_MAX_NORMAL) {
+                        if (bar->priv->base_volume < vol_max_normal) {
                                 str = g_strdup_printf ("<small>%s</small>", C_("volume", "100%"));
-                                gtk_scale_add_mark (GTK_SCALE (bar->priv->scale), ADJUSTMENT_MAX_NORMAL,
+                                gtk_scale_add_mark (GTK_SCALE (bar->priv->scale), vol_max_normal,
                                                     GTK_POS_BOTTOM, str);
                         }
                 }
@@ -558,7 +517,7 @@ gvc_channel_bar_set_base_volume (GvcChannelBar *bar,
         g_return_if_fail (GVC_IS_CHANNEL_BAR (bar));
 
         if (base_volume == 0) {
-                bar->priv->base_volume = ADJUSTMENT_MAX_NORMAL;
+                bar->priv->base_volume = gvc_mixer_control_get_vol_max_norm (bar->priv->mixer_control);
                 return;
         }
 
@@ -575,27 +534,15 @@ gvc_channel_bar_set_property (GObject       *object,
         GvcChannelBar *self = GVC_CHANNEL_BAR (object);
 
         switch (prop_id) {
+        case PROP_MIXER_CONTROL:
+                self->priv->mixer_control = g_value_dup_object (value);
+                break;
         case PROP_ORIENTATION:
                 gvc_channel_bar_set_orientation (self, g_value_get_enum (value));
                 break;
         case PROP_IS_MUTED:
                 gvc_channel_bar_set_is_muted (self, g_value_get_boolean (value));
                 break;
-        case PROP_ICON_NAME:
-                gvc_channel_bar_set_icon_name (self, g_value_get_string (value));
-                break;
-        case PROP_LOW_ICON_NAME:
-                gvc_channel_bar_set_low_icon_name (self, g_value_get_string (value));
-                break;
-        case PROP_HIGH_ICON_NAME:
-                gvc_channel_bar_set_high_icon_name (self, g_value_get_string (value));
-                break;
-        case PROP_ADJUSTMENT:
-                gvc_channel_bar_set_adjustment (self, g_value_get_object (value));
-                break;
-        case PROP_IS_AMPLIFIED:
-                gvc_channel_bar_set_is_amplified (self, g_value_get_boolean (value));
-                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -612,138 +559,51 @@ gvc_channel_bar_get_property (GObject     *object,
         GvcChannelBarPrivate *priv = self->priv;
 
         switch (prop_id) {
+        case PROP_MIXER_CONTROL:
+                g_value_set_object (value, priv->mixer_control);
+                break;
         case PROP_ORIENTATION:
                 g_value_set_enum (value, priv->orientation);
                 break;
         case PROP_IS_MUTED:
                 g_value_set_boolean (value, priv->is_muted);
                 break;
-        case PROP_ICON_NAME:
-                g_value_set_string (value, priv->icon_name);
-                break;
-        case PROP_LOW_ICON_NAME:
-                g_value_set_string (value, priv->low_icon_name);
-                break;
-        case PROP_HIGH_ICON_NAME:
-                g_value_set_string (value, priv->high_icon_name);
-                break;
-        case PROP_ADJUSTMENT:
-                g_value_set_object (value, gvc_channel_bar_get_adjustment (self));
-                break;
-        case PROP_IS_AMPLIFIED:
-                g_value_set_boolean (value, priv->is_amplified);
-                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
         }
 }
 
-static GObject *
-gvc_channel_bar_constructor (GType                  type,
-                             guint                  n_construct_properties,
-                             GObjectConstructParam *construct_params)
-{
-        GObject       *object;
-        GvcChannelBar *self;
-
-        object = G_OBJECT_CLASS (gvc_channel_bar_parent_class)->constructor (type, n_construct_properties, construct_params);
-
-        self = GVC_CHANNEL_BAR (object);
-
-        update_mute_switch (self);
-
-        return object;
-}
-
 static void
-gvc_channel_bar_class_init (GvcChannelBarClass *klass)
-{
-        GObjectClass   *object_class = G_OBJECT_CLASS (klass);
-
-        object_class->constructor = gvc_channel_bar_constructor;
-        object_class->finalize = gvc_channel_bar_finalize;
-        object_class->set_property = gvc_channel_bar_set_property;
-        object_class->get_property = gvc_channel_bar_get_property;
-
-        g_object_class_install_property (object_class,
-                                         PROP_ORIENTATION,
-                                         g_param_spec_enum ("orientation",
-                                                            "Orientation",
-                                                            "The orientation of the scale",
-                                                            GTK_TYPE_ORIENTATION,
-                                                            GTK_ORIENTATION_VERTICAL,
-                                                            G_PARAM_READWRITE));
-        g_object_class_install_property (object_class,
-                                         PROP_IS_MUTED,
-                                         g_param_spec_boolean ("is-muted",
-                                                               "is muted",
-                                                               "Whether stream is muted",
-                                                               FALSE,
-                                                               G_PARAM_READWRITE|G_PARAM_CONSTRUCT));
-
-        g_object_class_install_property (object_class,
-                                         PROP_ADJUSTMENT,
-                                         g_param_spec_object ("adjustment",
-                                                              "Adjustment",
-                                                              "The GtkAdjustment that contains the current value of this scale button object",
-                                                              GTK_TYPE_ADJUSTMENT,
-                                                              G_PARAM_READWRITE));
-        g_object_class_install_property (object_class,
-                                         PROP_ICON_NAME,
-                                         g_param_spec_string ("icon-name",
-                                                              "Icon Name",
-                                                              "Name of icon to display for this stream",
-                                                              NULL,
-                                                              G_PARAM_READWRITE|G_PARAM_CONSTRUCT));
-        g_object_class_install_property (object_class,
-                                         PROP_LOW_ICON_NAME,
-                                         g_param_spec_string ("low-icon-name",
-                                                              "Icon Name",
-                                                              "Name of icon to display for this stream",
-                                                              "audio-volume-low",
-                                                              G_PARAM_READWRITE|G_PARAM_CONSTRUCT));
-        g_object_class_install_property (object_class,
-                                         PROP_HIGH_ICON_NAME,
-                                         g_param_spec_string ("high-icon-name",
-                                                              "Icon Name",
-                                                              "Name of icon to display for this stream",
-                                                              "audio-volume-high",
-                                                              G_PARAM_READWRITE|G_PARAM_CONSTRUCT));
-        g_object_class_install_property (object_class,
-                                         PROP_IS_AMPLIFIED,
-                                         g_param_spec_boolean ("is-amplified",
-                                                               "Is amplified",
-                                                               "Whether the stream is digitally amplified",
-                                                               FALSE,
-                                                               G_PARAM_READWRITE|G_PARAM_CONSTRUCT));
-}
-
-static void
-gvc_channel_bar_init (GvcChannelBar *bar)
+gvc_channel_bar_constructed (GObject *object)
 {
+        GvcChannelBar *bar;
+        gdouble vol_max_normal;
         GtkWidget *frame;
 
-        bar->priv = gvc_channel_bar_get_instance_private (bar);
+        bar = GVC_CHANNEL_BAR (object);
+        G_OBJECT_CLASS (gvc_channel_bar_parent_class)->constructed (object);
 
-        bar->priv->base_volume = ADJUSTMENT_MAX_NORMAL;
+        vol_max_normal = gvc_mixer_control_get_vol_max_norm (bar->priv->mixer_control);
+
+        bar->priv->base_volume = vol_max_normal;
         bar->priv->low_icon_name = g_strdup ("audio-volume-low");
         bar->priv->high_icon_name = g_strdup ("audio-volume-high");
 
         bar->priv->orientation = GTK_ORIENTATION_VERTICAL;
         bar->priv->adjustment = GTK_ADJUSTMENT (gtk_adjustment_new (0.0,
                                                                     0.0,
-                                                                    ADJUSTMENT_MAX_NORMAL,
-                                                                    ADJUSTMENT_MAX_NORMAL/100.0,
-                                                                    ADJUSTMENT_MAX_NORMAL/10.0,
+                                                                    vol_max_normal,
+                                                                    vol_max_normal / 100.0,
+                                                                    vol_max_normal / 10.0,
                                                                     0.0));
         g_object_ref_sink (bar->priv->adjustment);
 
         bar->priv->zero_adjustment = GTK_ADJUSTMENT (gtk_adjustment_new (0.0,
                                                                          0.0,
-                                                                         ADJUSTMENT_MAX_NORMAL,
-                                                                         ADJUSTMENT_MAX_NORMAL/100.0,
-                                                                         ADJUSTMENT_MAX_NORMAL/10.0,
+                                                                         vol_max_normal,
+                                                                         vol_max_normal / 100.0,
+                                                                         vol_max_normal / 10.0,
                                                                          0.0));
         g_object_ref_sink (bar->priv->zero_adjustment);
 
@@ -775,6 +635,62 @@ gvc_channel_bar_init (GvcChannelBar *bar)
 
         gtk_container_add (GTK_CONTAINER (frame), bar->priv->scale_box);
         gtk_widget_show_all (frame);
+
+        update_mute_switch (bar);
+}
+
+static void
+gvc_channel_bar_dispose (GObject *object)
+{
+        GvcChannelBar *channel_bar;
+
+        channel_bar = GVC_CHANNEL_BAR (object);
+
+        g_clear_object (&channel_bar->priv->mixer_control);
+
+        G_OBJECT_CLASS (gvc_channel_bar_parent_class)->dispose (object);
+}
+
+static void
+gvc_channel_bar_class_init (GvcChannelBarClass *klass)
+{
+        GObjectClass   *object_class = G_OBJECT_CLASS (klass);
+
+        object_class->constructed = gvc_channel_bar_constructed;
+        object_class->dispose = gvc_channel_bar_dispose;
+        object_class->finalize = gvc_channel_bar_finalize;
+        object_class->set_property = gvc_channel_bar_set_property;
+        object_class->get_property = gvc_channel_bar_get_property;
+
+        g_object_class_install_property (object_class,
+                                         PROP_MIXER_CONTROL,
+                                         g_param_spec_object ("mixer-control",
+                                                              "mixer control",
+                                                              "mixer control",
+                                                              GVC_TYPE_MIXER_CONTROL,
+                                                              G_PARAM_READWRITE|G_PARAM_CONSTRUCT_ONLY));
+
+        g_object_class_install_property (object_class,
+                                         PROP_ORIENTATION,
+                                         g_param_spec_enum ("orientation",
+                                                            "Orientation",
+                                                            "The orientation of the scale",
+                                                            GTK_TYPE_ORIENTATION,
+                                                            GTK_ORIENTATION_VERTICAL,
+                                                            G_PARAM_READWRITE));
+        g_object_class_install_property (object_class,
+                                         PROP_IS_MUTED,
+                                         g_param_spec_boolean ("is-muted",
+                                                               "is muted",
+                                                               "Whether stream is muted",
+                                                               FALSE,
+                                                               G_PARAM_READWRITE|G_PARAM_CONSTRUCT));
+}
+
+static void
+gvc_channel_bar_init (GvcChannelBar *bar)
+{
+        bar->priv = gvc_channel_bar_get_instance_private (bar);
 }
 
 static void
@@ -797,10 +713,11 @@ gvc_channel_bar_finalize (GObject *object)
 }
 
 GtkWidget *
-gvc_channel_bar_new (void)
+gvc_channel_bar_new (GvcMixerControl *mixer_control)
 {
         GObject *bar;
         bar = g_object_new (GVC_TYPE_CHANNEL_BAR,
+                            "mixer-control", mixer_control,
                             "orientation", GTK_ORIENTATION_HORIZONTAL,
                             NULL);
         return GTK_WIDGET (bar);
