@@ -34,20 +34,21 @@
 
 struct GvcStreamStatusIconPrivate
 {
-        char          **icon_names;
-        GvcMixerStream *mixer_stream;
-        GtkWidget      *dock;
-        GtkWidget      *bar;
-        guint           current_icon;
-        char           *display_name;
-        gboolean        thaw;
+        GvcMixerControl  *mixer_control;
+
+        char            **icon_names;
+        GvcMixerStream   *mixer_stream;
+        GtkWidget        *dock;
+        GtkWidget        *bar;
+        guint             current_icon;
+        char             *display_name;
+        gboolean          thaw;
 };
 
 enum
 {
         PROP_0,
-        PROP_DISPLAY_NAME,
-        PROP_MIXER_STREAM,
+        PROP_MIXER_CONTROL,
         PROP_ICON_NAMES,
 };
 
@@ -490,7 +491,6 @@ gvc_stream_status_icon_set_display_name (GvcStreamStatusIcon *icon,
         g_free (icon->priv->display_name);
         icon->priv->display_name = g_strdup (name);
         update_icon (icon);
-        g_object_notify (G_OBJECT (icon), "display-name");
 }
 
 void
@@ -544,8 +544,6 @@ gvc_stream_status_icon_set_mixer_stream (GvcStreamStatusIcon *icon,
         }
 
         update_icon (icon);
-
-        g_object_notify (G_OBJECT (icon), "mixer-stream");
 }
 
 static void
@@ -557,11 +555,8 @@ gvc_stream_status_icon_set_property (GObject       *object,
         GvcStreamStatusIcon *self = GVC_STREAM_STATUS_ICON (object);
 
         switch (prop_id) {
-        case PROP_MIXER_STREAM:
-                gvc_stream_status_icon_set_mixer_stream (self, g_value_get_object (value));
-                break;
-        case PROP_DISPLAY_NAME:
-                gvc_stream_status_icon_set_display_name (self, g_value_get_string (value));
+        case PROP_MIXER_CONTROL:
+                self->priv->mixer_control = g_value_dup_object (value);
                 break;
         case PROP_ICON_NAMES:
                 gvc_stream_status_icon_set_icon_names (self, g_value_get_boxed (value));
@@ -582,11 +577,8 @@ gvc_stream_status_icon_get_property (GObject     *object,
         GvcStreamStatusIconPrivate *priv = self->priv;
 
         switch (prop_id) {
-        case PROP_MIXER_STREAM:
-                g_value_set_object (value, priv->mixer_stream);
-                break;
-        case PROP_DISPLAY_NAME:
-                g_value_set_string (value, priv->display_name);
+        case PROP_MIXER_CONTROL:
+                g_value_set_object (value, priv->mixer_control);
                 break;
         case PROP_ICON_NAMES:
                 g_value_set_boxed (value, priv->icon_names);
@@ -660,7 +652,7 @@ gvc_stream_status_icon_constructor (GType                  type,
         gtk_container_set_border_width (GTK_CONTAINER (box), 2);
         gtk_container_add (GTK_CONTAINER (frame), box);
 
-        icon->priv->bar = gvc_channel_bar_new ();
+        icon->priv->bar = gvc_channel_bar_new (icon->priv->mixer_control);
         gvc_channel_bar_set_orientation (GVC_CHANNEL_BAR (icon->priv->bar),
                                          GTK_ORIENTATION_VERTICAL);
 
@@ -684,6 +676,8 @@ gvc_stream_status_icon_dispose (GObject *object)
 {
         GvcStreamStatusIcon *icon = GVC_STREAM_STATUS_ICON (object);
 
+        g_clear_object (&icon->priv->mixer_control);
+
         if (icon->priv->dock != NULL) {
                 gtk_widget_destroy (icon->priv->dock);
                 icon->priv->dock = NULL;
@@ -709,19 +703,13 @@ gvc_stream_status_icon_class_init (GvcStreamStatusIconClass *klass)
         object_class->get_property = gvc_stream_status_icon_get_property;
 
         g_object_class_install_property (object_class,
-                                         PROP_MIXER_STREAM,
-                                         g_param_spec_object ("mixer-stream",
-                                                              "mixer stream",
-                                                              "mixer stream",
-                                                              GVC_TYPE_MIXER_STREAM,
-                                                              G_PARAM_READWRITE|G_PARAM_CONSTRUCT));
-        g_object_class_install_property (object_class,
-                                         PROP_DISPLAY_NAME,
-                                         g_param_spec_string ("display-name",
-                                                              "Display Name",
-                                                              "Name to display for this stream",
-                                                              NULL,
-                                                              G_PARAM_READWRITE|G_PARAM_CONSTRUCT));
+                                         PROP_MIXER_CONTROL,
+                                         g_param_spec_object ("mixer-control",
+                                                              "mixer control",
+                                                              "mixer control",
+                                                              GVC_TYPE_MIXER_CONTROL,
+                                                              G_PARAM_READWRITE|G_PARAM_CONSTRUCT_ONLY));
+
         g_object_class_install_property (object_class,
                                          PROP_ICON_NAMES,
                                          g_param_spec_boxed ("icon-names",
@@ -788,12 +776,12 @@ gvc_stream_status_icon_finalize (GObject *object)
 }
 
 GvcStreamStatusIcon *
-gvc_stream_status_icon_new (GvcMixerStream *stream,
-                            const char    **icon_names)
+gvc_stream_status_icon_new (GvcMixerControl  *control,
+                            const char      **icon_names)
 {
         GObject *icon;
         icon = g_object_new (GVC_TYPE_STREAM_STATUS_ICON,
-                             "mixer-stream", stream,
+                             "mixer-control", control,
                              "icon-names", icon_names,
                              NULL);
         return GVC_STREAM_STATUS_ICON (icon);
